@@ -40,8 +40,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
 __author__ = 'Alexandre Norman (norman@xael.org)'
-__version__ = '0.4.3'
-__last_modification__ = '2015.09.11'
+__version__ = '0.4.6'
+__last_modification__ = '2015.11.13'
 
 
 import collections
@@ -56,7 +56,6 @@ import sys
 import types
 from xml.etree import ElementTree as ET
 
-
 try:
     from multiprocessing import Process
 except ImportError:
@@ -161,8 +160,14 @@ class PortScanner(object):
         do not scan but interpret target hosts and return a list a hosts
         """
         assert type(hosts) is str, 'Wrong type for [hosts], should be a string [was {0}]'.format(type(hosts))
+        output = self.scan(hosts, arguments='-sL')
+        # Test if host was IPV6
+        try:
+            if 'looks like an IPv6 target specification' in output['nmap']['scaninfo']['error'][0]:
+                self.scan(hosts, arguments='-sL -6')
+        except KeyError:
+            pass
         
-        self.scan(hosts, arguments='-sL')
         return self.all_hosts()
 
 
@@ -185,17 +190,19 @@ class PortScanner(object):
         """
         if sys.version_info[0]==2:
             assert type(hosts) in (str, unicode), 'Wrong type for [hosts], should be a string [was {0}]'.format(type(hosts))
+            assert type(ports) in (str, unicode, type(None)), 'Wrong type for [ports], should be a string [was {0}]'.format(type(ports))
+            assert type(arguments) in (str, unicode), 'Wrong type for [arguments], should be a string [was {0}]'.format(type(arguments))
         else:
             assert type(hosts) is str, 'Wrong type for [hosts], should be a string [was {0}]'.format(type(hosts))
-        assert type(ports) in (str, type(None)), 'Wrong type for [ports], should be a string [was {0}]'.format(type(ports))
-        assert type(arguments) is str, 'Wrong type for [arguments], should be a string [was {0}]'.format(type(arguments))
+            assert type(ports) in (str, type(None)), 'Wrong type for [ports], should be a string [was {0}]'.format(type(ports))
+            assert type(arguments) is str, 'Wrong type for [arguments], should be a string [was {0}]'.format(type(arguments))
 
         for redirecting_output in ['-oX', '-oA']:
             assert not redirecting_output in arguments, 'Xml output can\'t be redirected from command line.\nYou can access it after a scan using:\nnmap.nm.get_nmap_last_output()'
 
         h_args = shlex.split(hosts)
         f_args = shlex.split(arguments)
-        
+
         # Launch scan
         args = [self._nmap_path, '-oX', '-'] + h_args + ['-p', ports]*(ports!=None) + f_args
         if sudo:
@@ -368,7 +375,7 @@ class PortScanner(object):
                 # reason
                 reason = dport.find('state').get('reason')
                 # name, product, version, extra info and conf if any
-                name,product,version,extrainfo,conf,cpe = '','','','','',''
+                name = product = version = extrainfo = conf = cpe = ''
                 for dname in dport.findall('service'):
                     name = dname.get('name')
                     if dname.get('product'):
@@ -385,15 +392,15 @@ class PortScanner(object):
                 # store everything
                 if not proto in list(scan_result['scan'][host].keys()):
                     scan_result['scan'][host][proto] = {}
-                    
+
                 scan_result['scan'][host][proto][port] = {'state': state,
-                                                  'reason': reason,
-                                                  'name': name,
-                                                  'product': product,
-                                                  'version': version,
-                                                  'extrainfo': extrainfo,
-                                                  'conf': conf,
-                                                  'cpe': cpe}
+                                                          'reason': reason,
+                                                          'name': name,
+                                                          'product': product,
+                                                          'version': version,
+                                                          'extrainfo': extrainfo,
+                                                          'conf': conf,
+                                                          'cpe': cpe}
                 script_id = ''
                 script_out = ''
                 # get script output if any
@@ -430,52 +437,47 @@ class PortScanner(object):
                 for dosmatch in dos.findall('osmatch'):
                     for dosclass in dosmatch.findall('osclass'):
                         # <osclass type="general purpose" vendor="Linux" osfamily="Linux" osgen="2.6.X" accuracy="98"/>
-                       ostype = ''
-                       vendor = ''
-                       osfamily = ''
-                       osgen = ''
-                       accuracy = ''
-                       try:
-                           ostype = dosclass.get('type')
-                           vendor = dosclass.get('vendor')
-                           osfamily = dosclass.get('osfamily')
-                           osgen = dosclass.get('osgen')
-                           accuracy = dosclass.get('accuracy')
-                       except AttributeError:
-                           pass
-       
-                       scan_result['scan'][host]['osclass'] = {
-                           'type': ostype,
-                           'vendor': vendor,
-                           'osfamily': osfamily,
-                           'osgen': osgen,
-                           'accuracy': accuracy
-                       }
-       
+                        ostype = ''
+                        vendor = ''
+                        osfamily = ''
+                        osgen = ''
+                        accuracy = ''
+                        try:
+                            ostype = dosclass.get('type')
+                            vendor = dosclass.get('vendor')
+                            osfamily = dosclass.get('osfamily')
+                            osgen = dosclass.get('osgen')
+                            accuracy = dosclass.get('accuracy')
+                        except AttributeError:
+                            pass
 
+                        scan_result['scan'][host]['osclass'] = {
+                            'type': ostype,
+                            'vendor': vendor,
+                            'osfamily': osfamily,
+                            'osgen': osgen,
+                            'accuracy': accuracy
+                        }
 
-            for dport in dhost.findall('osclass'):
-                # <osclass name="Linux 2.6.31" accuracy="98" line="30043"/>
-                name = ''
-                accuracy = ''
-                line = ''
-                try:
-                    name = dport.get('name')
-                    accuracy = dport.get('accuracy')
-                    line = dport.get('line')
-                except AttributeError:
-                    pass
-                if not 'osclass' in list(scan_result['scan'][host].keys()):
-                    scan_result['scan'][host]['osclass'] = []
+                for dport in dos.findall('osmatch'):
+                    # <osmatch name="Linux 3.7 - 3.15" accuracy="100" line="52790">
+                    name = ''
+                    accuracy = ''
+                    line = ''
+                    try:
+                        name = dport.get('name')
+                        accuracy = dport.get('accuracy')
+                        line = dport.get('line')
+                    except AttributeError:
+                        pass
+                    if not 'osmatch' in list(scan_result['scan'][host].keys()):
+                        scan_result['scan'][host]['osmatch'] = {}
 
-                scan_result['scan'][host]['osclass'].append(
-                    {
+                    scan_result['scan'][host]['osmatch'] = {
                         'name': name,
                         'accuracy': accuracy,
                         'line': line,
-                        }
-                    )
-
+                    }
 
             for dport in dhost.findall('osfingerprint'):
                 # <osfingerprint fingerprint="OS:SCAN(V=5.50%D=11/[...]S)&#xa;"/>
@@ -640,13 +642,13 @@ def __scan_progressive__(self, hosts, ports, arguments, callback, sudo):
 
 ############################################################################
 
+
 class PortScannerAsync(object):
     """
     PortScannerAsync allows to use nmap from python asynchronously
     for each host scanned, callback is called with scan result for the host
 
     """
-
     def __init__(self):
         """
         Initialize the module
@@ -659,7 +661,6 @@ class PortScannerAsync(object):
         self._nm = PortScanner()
         return
 
-
     def __del__(self):
         """
         Cleanup when deleted
@@ -669,7 +670,6 @@ class PortScannerAsync(object):
             self._process.terminate()
         return
 
-
     def scan(self, hosts='127.0.0.1', ports=None, arguments='-sV', callback=None, sudo=False):
         """
         Scan given hosts in a separate process and return host by host result using callback function
@@ -683,9 +683,15 @@ class PortScannerAsync(object):
         :param sudo: launch nmap with sudo if true
         """
 
-        assert type(hosts) is str, 'Wrong type for [hosts], should be a string [was {0}]'.format(type(hosts))
-        assert type(ports) in (str, type(None)), 'Wrong type for [ports], should be a string [was {0}]'.format(type(ports))
-        assert type(arguments) is str, 'Wrong type for [arguments], should be a string [was {0}]'.format(type(arguments))
+        if sys.version_info[0]==2:
+            assert type(hosts) in (str, unicode), 'Wrong type for [hosts], should be a string [was {0}]'.format(type(hosts))
+            assert type(ports) in (str, unicode, type(None)), 'Wrong type for [ports], should be a string [was {0}]'.format(type(ports))
+            assert type(arguments) in (str, unicode), 'Wrong type for [arguments], should be a string [was {0}]'.format(type(arguments))
+        else:
+            assert type(hosts) is str, 'Wrong type for [hosts], should be a string [was {0}]'.format(type(hosts))
+            assert type(ports) in (str, type(None)), 'Wrong type for [ports], should be a string [was {0}]'.format(type(ports))
+            assert type(arguments) is str, 'Wrong type for [arguments], should be a string [was {0}]'.format(type(arguments))
+            
         assert callable(callback) or callback is None, 'The [callback] {0} should be callable or None.'.format(str(callback))
 
         for redirecting_output in ['-oX', '-oA']:
@@ -700,7 +706,6 @@ class PortScannerAsync(object):
         self._process.start()
         return
 
-
     def stop(self):
         """
         Stop the current scan process
@@ -710,7 +715,6 @@ class PortScannerAsync(object):
             self._process.terminate()
         return
 
-
     def wait(self, timeout=None):
         """
         Wait for the current scan process to finish, or timeout
@@ -718,14 +722,11 @@ class PortScannerAsync(object):
         :param timeout: default = None, wait timeout seconds 
 
         """
-
         assert type(timeout) in (int, type(None)), 'Wrong type for [timeout], should be an int or None [was {0}]'.format(type(timeout))
 
         self._process.join(timeout)
         return
 
-    
-
     def still_scanning(self):
         """
         :returns: True if a scan is currently running, False otherwise
@@ -736,7 +737,6 @@ class PortScannerAsync(object):
         except:
             return False
 
-    
 
 ############################################################################
 
